/* 
* Copyright (C) 2024 Dominik Meyer
*  
* This program is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
*  
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*  
* You should have received a copy of the GNU General Public License
* along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/
#include "nlohmann/json_fwd.hpp"
#include <Redmine/API.hpp>
#include <Redmine/Exception/Api.hpp>
#include <exception>
#include <memory>
#include <stdexcept>
#include <vector>
#define LOGURU_WITH_STREAMS 1
#include <loguru.hpp>
#include <httplib.h>

Redmine::API::API(const std::string &serverURL, const std::string &authToken)
{

    if (serverURL.empty() || serverURL.size() < 7)
    {
        DLOG_S(ERROR) << "server URL not valid";

        throw std::invalid_argument("server url invalid");
    }

    if (!serverURL.starts_with("https://"))
    {
        DLOG_S(ERROR) << "server URL not starting with https://";

        throw std::invalid_argument("server URL does not start with https://");
    }

    redmineApiURL_=serverURL;

    if (authToken.empty() || authToken.length() < 10)
    {
        DLOG_S(ERROR) << "auth token not valid";

        throw std::invalid_argument("auth token is not valid");
    }   


    authToken_=authToken;
}

 nlohmann::json Redmine::API::get(const std::string  &path) const
 {
    DLOG_S(INFO) << "getting API endpoint " << path+".json" << " from " <<redmineApiURL_;
    httplib::Client client{redmineApiURL_};

    httplib::Headers headers = 
    {
        { "X-Redmine-API-Key", authToken_ },
        { "Content-Type", "application/json"}
    };

    auto res = client.Get(path + ".json" , headers);

    if (res->status == httplib::StatusCode::Unauthorized_401)
    {
        DLOG_S(ERROR) << "authentication with API server failed (401)";
        throw std::runtime_error("authentication failed");
    }
    else if (res->status == httplib::StatusCode::NotFound_404)
    {
        DLOG_S(ERROR) << "unknown API endpoint (404)";
        throw std::runtime_error("unknown API endpoint");
    }
    else if (res->status != httplib::StatusCode::OK_200)
    {
        LOG_S(ERROR) << "unsupported error " << httplib::status_message(res->status);
        throw std::runtime_error("unsupported error");
    }

    DLOG_S(INFO) << "get successful";
    return nlohmann::json::parse(res->body);
 }

void Redmine::API::put(const std::string  &path, const nlohmann::json &data) const
 {
    DLOG_S(INFO) << "putting API endpoint " << path+".json" << " from " <<redmineApiURL_;
    httplib::Client client{redmineApiURL_};
    client.set_basic_auth(authToken_, "");

    auto res = client.Put(path + ".json", data.dump(),"application/json");

    if (res->status == httplib::StatusCode::Unauthorized_401)
    {
        DLOG_S(ERROR) << "authentication with API server failed (401)";
        throw std::runtime_error("authentication failed");
    }
    else if (res->status == httplib::StatusCode::NotFound_404)
    {
        DLOG_S(ERROR) << "unknown API endpoint (404)";
        throw std::runtime_error("unknown API endpoint");
    }
    else if (res->status == httplib::StatusCode::UnprocessableContent_422)
    {
        nlohmann::json error=nlohmann::json::parse(res->body);
        std::vector<std::string> errors=error["errors"].get<std::vector<std::string>>();
        throw Redmine::Exception::Api("Unprocessible Content",errors);
    }
    else if (res->status != httplib::StatusCode::OK_200 && res->status != httplib::StatusCode::NoContent_204)
    {
        LOG_S(ERROR) << "unsupported error " << httplib::status_message(res->status);
        LOG_S(ERROR) << res->body;
        throw std::runtime_error("unsupported error");
    }
    
    DLOG_S(INFO) << "put successful";
 }


Redmine::User Redmine::API::getMyAccount() const 
{
    nlohmann::json userInfo = get("/my/account");

    return Redmine::User{userInfo};
}

void Redmine::API::setMyAccount(const Redmine::User &user) const
{

    put("/my/account", user.get());

}


bool Redmine::API::ready() const 
{   
    try {
        getMyAccount();
    } catch (std::exception &e)
    {
        return false;
    }
    return true;
}